function Model=crossfit(X,Dtime,Dtreat,Rglobal,Rlocal,times,treatments,samples,variables);
%   Model generating Global, Local and Crossfit models of crossed data with
%   two factors, time and treatment. In this program the factor treatment
%   is left intact, while separate local models are fitted for different
%   time-points. Treatment and time can be replaced by other factors at
%   will.
%
%   Input:
%   X           (m x nvars)     Matrix containing the data: 'm' samples and 'nvars' variables
%   Dtime       (m x ntimes)    Design Matrix describing to which of time-point 1...itime...ntimes, sample 'm' belongs
%   Dtreat      (m x ntreats)   Design Matrix describing to which of treatment 1...itreat...ntreats, sample 'm' belongs
%   Rglobal     (1)             # of components for the global model
%   Rlocal      (1 x ntimes)    # of components for the local model at timepoint 'itime'
%   times       (1 x ntimes)    labels for all timepoints
%   treatments  (1 x ntreats)   labels for all treatments
%   samples     (1 x m)         sample labels
%   variables   (1 x nvars)     variable labels
%   
%   Output:
%   Model       Model Structure
%   Model.Global    (1)                Global Model Structure
%   Model.Local     (1 x ntimes)       Local Model Structure of length 'ntimes'
%   Model.Crossfit  (ntimes x ntimes)  Crossfit Model Structure
%   SVD                                SVD output structure, containing X=USV'
%   
%   '*' denotes 'Global','Local' or 'Crossfit' where appropriate
%   Model.*.X          (m x nvars) matrix containing centered data for global model
%   Model.*.Means      (ntimes x nvars) matrix containing means per day
%   Model.*.Dtreat     (m x ntreats) design matrix of treatments
%   Model.*.Dtime      (m x ntimes) design matrix of times
%   Model.*.Samples    (m x 1) vector containing sample labels
%   Model.*.times      (ntimes x 1) vector containing time labels
%   Model.*.treatments (ntreats x 1) vector containing treatment labels
%   Model.*.variables  (nvars x 1) vector containing variable labels
%   Model.*.Scores     (m x Rglobal) matrix containing global scores
%   Model.*.Loadings   (nvars x Rglobal) matrix containing global loadings
%   Model.*.SSQ        (1 x Rglobal) vector containing variation explained by each global component
%   Model.*.SSQtot     (1) variation explained by entire global model
%   Model.*.PerDay     (1 x ntimes) structure containing global model and diagnostics per time-point

%%  Define # of components when not defined
if isempty(Rglobal)
    Rglobal=min(size(X,1)-size(Dtime,2),size(X,2));
    nperday=sum(Dtime);
end
if isempty(Rlocal)
    for iday=1:size(Dtime,2)
        Rlocal(iday)=min(nperday(iday)-1,size(X,2));
    end
end

%%  Arrange and preprocess data
Model.Global.X=[];
Model.Global.Means=[];
Model.Global.Dtreat=[];
Model.Global.Dtime=[];
Model.Global.Samples=[];

for iday=1:size(Dtime,2)
    ismp=find(Dtime(:,iday));
    [Xnow,meannow]=mncn(X(ismp,:));
    Dtreatnow=Dtreat(ismp,:);
    Dtimenow=Dtime(ismp,:);
    
    Model.Global.X=[Model.Global.X;Xnow];
    Model.Global.Means=[Model.Global.Means;meannow];
    Model.Global.Dtime=[Model.Global.Dtime;Dtimenow];
    Model.Global.Dtreat=[Model.Global.Dtreat;Dtreatnow];
    Model.Global.times=times;
    Model.Global.treatments=treatments;
    Model.Global.Samples=[Model.Global.Samples;samples(ismp)];
    Model.Global.variables=variables;
    
    Model.Local{iday}.X=Xnow;
    Model.Local{iday}.mean=meannow;
    Model.Local{iday}.Dtreat=Dtreatnow;
    Model.Local{iday}.time=times(iday);
    Model.Local{iday}.treatments=treatments;
    Model.Local{iday}.vamples=samples(ismp);
    Model.Local{iday}.variables=variables;
end
clear iday ismp
%%  Generate Global and Local Models
%   Global Model
[Model.Global.SVD.U,Model.Global.SVD.S,Model.Global.SVD.V]=svd(Model.Global.X,'econ');
Model.Global.Scores=Model.Global.SVD.U(:,1:Rglobal)*Model.Global.SVD.S(1:Rglobal,1:Rglobal);
Model.Global.Loadings=Model.Global.SVD.V(:,1:Rglobal);
Model.Global.SSQ=diag(Model.Global.SVD.S(1:Rglobal,1:Rglobal).^2)./sum(diag(Model.Global.SVD.S.^2))*100;
Model.Global.SSQtot=sum(Model.Global.SSQ);

for iday=1:size(Dtime,2)
    ismp=find(Dtime(:,iday));
    Model.Global.PerDay{iday}.X=Model.Global.X(ismp,:);
    Model.Global.PerDay{iday}.Scores=Model.Global.Scores(ismp,:);
    Model.Global.PerDay{iday}.Samples=Model.Global.Samples(ismp,:);
    Model.Global.PerDay{iday}.Dtreat=Model.Global.Dtreat(ismp,:);
    for irglobal=1:Rglobal;
        Model.Global.PerDay{iday}.SSQperpc(irglobal)=(ssq(Model.Global.PerDay{iday}.Scores(:,irglobal)*Model.Global.Loadings(:,irglobal)')./ssq(Model.Global.PerDay{iday}.X))*100;
    end
    Model.Global.PerDay{iday}.SSQtot=(ssq(Model.Global.PerDay{iday}.Scores(:,1:Rglobal)*Model.Global.Loadings(:,1:Rglobal)')./ssq(Model.Global.PerDay{iday}.X))*100;
end
clear iday ismp
%   Local Model
for iday=1:size(Dtime,2)
    [Model.Local{iday}.SVD.U,Model.Local{iday}.SVD.S,Model.Local{iday}.SVD.V]=svd(Model.Local{iday}.X,'econ');
    Model.Local{iday}.Scores=Model.Local{iday}.SVD.U(:,1:Rlocal(iday))*Model.Local{iday}.SVD.S(1:Rlocal(iday),1:Rlocal(iday));
    Model.Local{iday}.Loadings=Model.Local{iday}.SVD.V(:,1:Rlocal(iday));
    Model.Local{iday}.SSQ=diag(Model.Local{iday}.SVD.S(1:Rlocal(iday),1:Rlocal(iday)).^2)./sum(diag(Model.Local{iday}.SVD.S).^2)*100;
    Model.Local{iday}.SSQtot=(ssq(Model.Local{iday}.Scores(:,1:Rlocal(iday))*Model.Local{iday}.Loadings(:,1:Rlocal(iday))')./ssq(Model.Local{iday}.X))*100;
end
clear iday
%%  Crossfit
for idata=1:length(Model.Local)
    for imodel=1:length(Model.Local)
        Model.Crossfit{idata,imodel}.X=Model.Local{idata}.X;
        Model.Crossfit{idata,imodel}.Loadings=Model.Local{imodel}.Loadings;
        Model.Crossfit{idata,imodel}.Scores=Model.Crossfit{idata,imodel}.X*Model.Crossfit{idata,imodel}.Loadings;
        for irlocal=1:Rlocal(imodel);
            Model.Crossfit{idata,imodel}.SSQperpc(irlocal)=(ssq(Model.Crossfit{idata,imodel}.Scores(:,irlocal)*Model.Crossfit{idata,imodel}.Loadings(:,irlocal)')./ssq(Model.Crossfit{idata,imodel}.X))*100;
        end
        Model.Crossfit{idata,imodel}.SSQtot=(ssq(Model.Crossfit{idata,imodel}.Scores(:,1:Rlocal(imodel))*Model.Crossfit{idata,imodel}.Loadings(:,1:Rlocal(imodel))')./ssq(Model.Crossfit{idata,imodel}.X))*100;
    end
end   
